﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml;

namespace UrlMgr
{
    /// <summary>
    /// 配置文件操作类
    /// </summary>
    public class Config:BaseClass
    {
        /// <summary>
        /// 缓存配置参数
        /// </summary>
        private static Dictionary<string, string> dic = null;

        /// <summary>
        /// 配置文件xml的路径
        /// </summary>
        private static readonly string ConfigPath = AppDomain.CurrentDomain.BaseDirectory + "Config\\MySet.xml";
 
        /// <summary>
        /// 获得配置信息，使用缓存数据
        /// </summary>
        /// <param name="key"></param>
        /// <param name="catalog"></param>
        /// <returns></returns>
        public string this[string fieldName, string nodePath] 
        {
            get
            {// 读取配置参数
                string result = ""; 
                if(dic != null && dic.Count > 0)
                {
                    if(dic.Keys.Contains(fieldName))
                    {
                        result = dic[fieldName];
                    }
                    else
                    {
                        result = this.GetValue(fieldName, nodePath);
                        dic.Add(fieldName, result);
                    }
                }
                else
                {
                    if(dic == null)
                    {
                        dic = new Dictionary<string,string>();
                    }

                    result = this.GetValue(fieldName, nodePath);
                    dic.Add(fieldName, result);
                }
                return result;
            }
            set
            {// 修改配置参数
                this.SetValue(fieldName, nodePath, value);
            }
        }

        /// <summary>
        /// 查询配置节点的参数
        /// </summary>
        /// <param name="fieldName">字段名称</param>
        /// <param name="nodePath">字段路径</param>
        /// <returns></returns>
        private string GetValue(string fieldName, string nodePath)
        { 
            string result = "";
            XmlDocument xdoc = null;
            XmlNode node = null;
            if (!File.Exists(ConfigPath))
            {
                throw new Exception("Config\\MySet.config is not exsist");
            }

            try
            {
                xdoc = new XmlDocument();
                xdoc.Load(ConfigPath);
                if(xdoc != null)
                {
                    // node = xdoc.SelectSingleNode(string.Format("//Config/BaseInfo/Node[@id='{0}' and @Enable = '1']", fieldName));
                    node = xdoc.SelectSingleNode(string.Format("{0}[@ID = '{1}']", nodePath, fieldName));
                    if(node != null)
                    {
                        result = node.InnerText;
                    }
                }
            }
            catch(Exception ex)
            {
                throw ex;
            }
            finally
            {
                xdoc = null;
                node = null;
            } 
            return result; 
        }

        /// <summary>
        /// 修改配置节点的参数
        /// </summary>
        /// <param name="fieldName">字段名称</param>
        /// <param name="nodePath">字段路径</param>
        /// <returns></returns>
        private bool SetValue(string fieldName, string nodePath, string fieldValue)
        { 
            bool result = false;
            XmlDocument xdoc = null;
            XmlNode node = null;
            if (!File.Exists(ConfigPath))
            {
                throw new Exception("Config\\WebPartMaker.config is not exsist");
            }

            try
            {
                xdoc = new XmlDocument();
                xdoc.Load(ConfigPath);
                if(xdoc != null)
                {
                    // node = xdoc.SelectSingleNode(string.Format("//Config/BaseInfo/Node[@id='{0}' and @Enable = '1']", fieldName));
                    node = xdoc.SelectSingleNode(string.Format("{0}[@ID='{1}']", nodePath, fieldName));
                    if(node != null)
                    {
                        node.InnerText = fieldValue;
                        xdoc.Save(ConfigPath);
                        result = true;
                    }
                }
            }
            catch(Exception ex)
            {
                throw ex;
            }
            finally
            {
                xdoc = null;
                node = null;
            } 
            return result; 
        }

    }

    /// <summary>
    /// 系统配置参数工具类
    /// </summary>
    public static class MySettings
    {
        private static Config myConfig = new Config();  

        #region XmlDoc Parameters

        /// <summary>
        /// XmlData存放路径
        /// </summary>
        public static readonly string XmlDataFpath = System.AppDomain.CurrentDomain.BaseDirectory + "XmlData\\";

        #region Catalog.xml Fields

        /// <summary>
        /// Catalog.xml文件路径
        /// </summary>
        public static readonly string CatalogXmlFpath = XmlDataFpath + "Catalog.xml";

        /// <summary>
        /// 分类信息xml节点路径
        /// </summary>
        private static readonly string CatalogInfoPath = "//Config/XmlDocInfo/CatalogInfo/Node"; 

        /// <summary>
        /// 主键
        /// </summary>
        public static readonly string Catalog_ID = myConfig["Catalog_ID", CatalogInfoPath];

        /// <summary>
        /// 分类名称
        /// </summary>
        public static readonly string Catalog_Title = myConfig["Catalog_Title", CatalogInfoPath];

        /// <summary>
        /// 分类描述
        /// </summary>
        public static readonly string Catalog_Desc = myConfig["Catalog_Desc", CatalogInfoPath];

        /// <summary>
        /// 创建者
        /// </summary>
        public static readonly string Catalog_Creator = myConfig["Catalog_Creator", CatalogInfoPath];

        /// <summary>
        /// 创建时间
        /// </summary>
        public static readonly string Catalog_CreateTime = myConfig["Catalog_CreateTime", CatalogInfoPath];

        /// <summary>
        /// 修改人
        /// </summary>
        public static readonly string Catalog_Editor = myConfig["Catalog_Editor", CatalogInfoPath];

        /// <summary>
        /// 修改时间
        /// </summary>
        public static readonly string Catalog_EditTime = myConfig["Catalog_EditTime", CatalogInfoPath]; 

        #endregion

        #region UrlNode.xml Fields

        /// <summary>
        /// UrlNode.xml文件路径
        /// </summary>
        public static readonly string UrlNodeXmlFpath = XmlDataFpath + "UrlNode.xml";

        /// <summary>
        /// 分类信息xml节点路径
        /// </summary>
        private static readonly string UrlNodeInfoPath = "//Config/XmlDocInfo/UrlNodeInfo/Node";

        /// <summary>
        /// 主键
        /// </summary>
        public static readonly string UrlNode_ID = myConfig["UrlNode_ID", UrlNodeInfoPath];

        /// <summary>
        /// 分类主键
        /// </summary>
        public static readonly string UrlNode_CatalogID = myConfig["UrlNode_CatalogID", UrlNodeInfoPath];

        /// <summary>
        /// 上级节点
        /// </summary>
        public static readonly string UrlNode_ParentID = myConfig["UrlNode_ParentID", UrlNodeInfoPath];

        /// <summary>
        /// 顺序编码
        /// </summary>
        public static readonly string UrlNode_SortCode = myConfig["UrlNode_SortCode", UrlNodeInfoPath]; 

        /// <summary>
        /// 链接名称
        /// </summary>
        public static readonly string UrlNode_Title = myConfig["UrlNode_Title", UrlNodeInfoPath];

        /// <summary>
        /// 链接描述
        /// </summary>
        public static readonly string UrlNode_Desc = myConfig["UrlNode_Desc", UrlNodeInfoPath];

        /// <summary>
        /// 链接地址
        /// </summary>
        public static readonly string UrlNode_Url = myConfig["UrlNode_Url", UrlNodeInfoPath]; 

        /// <summary>
        /// 创建者
        /// </summary>
        public static readonly string UrlNode_Creator = myConfig["UrlNode_Creator", UrlNodeInfoPath];

        /// <summary>
        /// 创建时间
        /// </summary>
        public static readonly string UrlNode_CreateTime = myConfig["UrlNode_CreateTime", UrlNodeInfoPath];

        /// <summary>
        /// 修改人
        /// </summary>
        public static readonly string UrlNode_Editor = myConfig["UrlNode_Editor", UrlNodeInfoPath];

        /// <summary>
        /// 修改时间
        /// </summary>
        public static readonly string UrlNode_EditTime = myConfig["UrlNode_EditTime", UrlNodeInfoPath];

        /// <summary>
        /// 备注信息
        /// </summary>
        public static readonly string UrlNode_Demo = myConfig["UrlNode_Demo", UrlNodeInfoPath];

        #endregion

        #endregion

        #region Config Parameters

        /// <summary>
        /// Config存放路径
        /// </summary>
        public static readonly string ConfigFpath = AppDomain.CurrentDomain.BaseDirectory + "Config\\";

        #region MySet.xml Fields

        /// <summary>
        /// MySet.xml存放路径
        /// </summary>
        public static readonly string MySetXmlFpath = ConfigFpath + "MySet.xml";

        /// <summary>
        /// 分类信息xml节点路径
        /// </summary>
        private static readonly string MySetInfoPath = "//Config/BasicSetInfo/Node";

        /// <summary>
        /// 默认显示的分类信息的主键
        /// </summary>
        public static string MySet_DefaultCatalogID
        {
            get { return myConfig["DefaultCatalogID", MySetInfoPath]; }
            set { myConfig["DefaultCatalogID", MySetInfoPath] = value; }
        } 

        /// <summary>
        /// 长字符时间格式, yyyy/MM/dd HH:mm:ss
        /// </summary>
        public static readonly string TimeLong24HHFormat = myConfig["TimeLong24HHFormat", MySetInfoPath];

        /// <summary>
        /// 历史查询天数
        /// </summary>
        public static readonly int QueryHistoryDay = int.Parse(myConfig["QueryHistoryDay", MySetInfoPath]);

        #endregion
        #endregion

        #region Project Settings 
        
        #endregion
    }


}
