﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace UrlMgr
{
    public class BaseClass:IDisposable
    {
        public void Dispose()
        {
        } 
    }

    public class CoreViewState:BaseClass
    {
        private Dictionary<string, object> viewstate = new Dictionary<string, object>();

        /// <summary>
        /// 获取key字典数据
        /// </summary>
        /// <param name="catalog"></param>
        /// <returns></returns>
        public object this[string key]
        {
            get
            {
                if (viewstate != null && viewstate.Keys.Count > 0 && viewstate.ContainsKey(key) && viewstate[key] != null)
                {
                    return viewstate[key];
                }
                else
                {
                    return null;
                }
            }
            set
            {
                if (viewstate == null)
                {
                    viewstate = new Dictionary<string, object>();
                }

                if (!viewstate.ContainsKey(key))
                {
                    viewstate.Add(key, null);
                }

                viewstate[key] = value;
            }
        }

        /// <summary>
        /// 判断键值是否存在
        /// </summary>
        /// <param name="catalog"></param> 
        /// <returns></returns>
        public bool Containts(string key)
        {
            if (viewstate != null
                && viewstate.Count > 0
                && viewstate.ContainsKey(key))
            {
                return viewstate.ContainsKey(key);
            }

            return false;
        }

        /// <summary>
        /// 窗体缓存的变量数目
        /// </summary>
        public int Count
        {
            get
            {
                if (viewstate != null)
                {
                    return viewstate.Count;
                }

                return 0;
            }
        }
    }

    public class BaseForm : Form, IDisposable
    { 
        protected CoreViewState ViewState = new CoreViewState();

        protected Action MyAction = new Action();

        public void Dispose()
        {
            ViewState.Dispose();
            MyAction.Dispose();
            base.Dispose();
        }
    }

    public class Catalog:BaseClass
    {
        public string ID
        {
            get;
            set;
        }

        public string Title
        {
            get;
            set;
        }

        public string Desc
        {
            get;
            set;
        }

        public string Creator
        {
            get;
            set;
        }

        public DateTime CreateTime
        {
            get;
            set;
        }

        public string Editor
        {
            get;
            set;
        }

        public DateTime EditTime
        {
            get;
            set;
        }
    }

    public class UrlNode : BaseClass
    {
        public string ID
        {
            get;
            set;
        }

        public string CatalogID
        {
            get;
            set;
        }

        public string ParentID
        {
            get;
            set;
        }

        public string SortCode
        {
            get;
            set;
        }

        public string Title
        {
            get;
            set;
        }

        public string Desc
        {
            get;
            set;
        }

        public string Url
        {
            get;
            set;
        }

        public string Creator
        {
            get;
            set;
        }

        public DateTime CreateTime
        {
            get;
            set;
        }

        public string Editor
        {
            get;
            set;
        }

        public DateTime EditTime
        {
            get;
            set;
        }

        public string Demo
        {
            get;
            set;
        }

        /// <summary>
        /// 查询条件 开始时间
        /// </summary>
        public DateTime BeginTime
        {
            get;
            set;
        }

        /// <summary>
        /// 查询条件 结束时间
        /// </summary>
        public DateTime EndTime
        {
            get;
            set;
        }
    }
}
