﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using UrlMgr;

namespace UrlMgr
{
    public partial class CatalogMgr : BaseForm
    {
        /// <summary>
        /// 当前对象
        /// </summary>
        private Catalog curCatalog
        {
            get
            {
                return ViewState["curCatalog"] as Catalog; 
            }
            set
            {
                ViewState["curCatalog"] = value;
            }
        }

        /// <summary>
        /// 查询条件
        /// </summary>
        private Catalog curWhere
        {
            get
            {
                return ViewState["curQuery"] as Catalog; 
            }
            set
            {
                ViewState["curQuery"] = value;
            }
        }

        /// <summary>
        /// Catalog查询结果缓存
        /// </summary>
        private List<Catalog> curQuery
        {
            get
            {
                return ViewState["curQuery"] as List<Catalog>;
            }
            set
            {
                ViewState["curQuery"] = value;
            }
        }

        public CatalogMgr()
        {
            InitializeComponent();
        } 

        private void CatalogMgr_Load(object sender, EventArgs e)
        {
            this.BindCatalog(true);
        }

        public void BindCatalog(bool refrash)
        {
            List<Catalog> list = null;
            if (refrash)
            {// 刷新
                list = MyAction.GetCatalog(this.curWhere);
                this.curQuery = list;
            }
            else
            {
                list = this.curQuery;
            }

            dgvCatalog.DataSource = list;
            if (list == null || list.Count == 0)
            {
                MessageBox.Show("查询结果为空！", "系统提示", MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1);
            } 
        }

        private void btnGet_Click(object sender, EventArgs e)
        {
            this.curWhere = new Catalog
            {
                Title = txtTitle.Text.Trim(),
                Desc = txtDesc.Text.Trim()
            };
            // Data Refresh
            this.BindCatalog(true);
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            int flag = 0;
            this.curCatalog = new Catalog()
            {
                ID = Guid.NewGuid().ToString(),
                Title = txtTitle.Text.Trim(),
                Desc = txtDesc.Text.Trim(),
                Creator = System.Environment.UserName,
                CreateTime = DateTime.Now, 
            };
            flag = MyAction.AddCatalog(this.curCatalog);
            if (flag > 0)
            {
                MessageBox.Show("添加成功！", "系统提示", MessageBoxButtons.OK);
                this.BindCatalog(true);
            }
            else
            {
                MessageBox.Show("添加失败！", "系统提示", MessageBoxButtons.OK);
                this.BindCatalog(false);
            }

            this.curCatalog = null;
        }

        private void btnDel_Click(object sender, EventArgs e)
        {
            int flag = 0;
            if (this.curCatalog == null)
            {
                MessageBox.Show("请先选择要修改的数据", "系统提示");
                return;
            }

            this.curCatalog = new Catalog()
            {
                ID = this.curCatalog.ID
            };

            flag = MyAction.DelCatalog(this.curCatalog);
            if (flag > 0)
            {
                MessageBox.Show("删除成功！", "系统提示", MessageBoxButtons.OK);
                this.BindCatalog(true);
            }
            else
            {
                MessageBox.Show("删除失败！", "系统提示", MessageBoxButtons.OK);
                this.BindCatalog(false);
            } 
        }

        private void btnSet_Click(object sender, EventArgs e)
        {
            int flag = 0;
            if (this.curCatalog == null)
            {
                MessageBox.Show("请先选择要修改的数据", "系统提示");
                return;
            }

            this.curCatalog = new Catalog()
            {
                ID = this.curCatalog.ID,
                Title = txtTitle.Text,
                Desc = txtDesc.Text,
                Editor = System.Environment.UserName,
                EditTime = DateTime.Now
            };

            flag = MyAction.SetCatalog(this.curCatalog);
            if (flag > 0)
            {
                MessageBox.Show("修改成功！", "系统提示", MessageBoxButtons.OK);
                this.BindCatalog(true);
            }
            else
            {
                MessageBox.Show("修改失败！", "系统提示", MessageBoxButtons.OK);
                this.BindCatalog(false);
            }
        }

        private void dgvCatalog_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                this.curCatalog = new Catalog()
                {
                    ID = dgvCatalog.Rows[e.RowIndex].Cells[0].Value.ToString().Trim(),
                    Title = dgvCatalog.Rows[e.RowIndex].Cells[1].Value.ToString().Trim(),
                    Desc = dgvCatalog.Rows[e.RowIndex].Cells[2].Value.ToString().Trim(), 
                };

                txtTitle.Text = this.curCatalog.Title;
                txtDesc.Text = this.curCatalog.Desc;
            }
        }
    }
}
