﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace UrlMgr
{
    public partial class UrlNodeMgr : BaseForm
    { 
        private UrlNode curUrlNode
        {
            get
            {
                return ViewState["curUrlNode"] as UrlNode;
            }
            set
            {
                ViewState["curUrlNode"] = value;
            }
        }

        private UrlNode curWhere
        {
            get
            {
                return ViewState["curWhere"] as UrlNode;
            }
            set
            {
                ViewState["curWhere"] = value;
            }
        }

        private List<UrlNode> curQuery
        {
            get
            {
                return ViewState["curQuery"] as List<UrlNode>;
            }
            set
            {
                ViewState["curQuery"] = value;
            }
        }

        public bool ReceiveMessage(string key, object value)
        {
            bool flag = false;
            switch (key)
            {
                case "curUrlNode":
                    {
                        this.curUrlNode = value as UrlNode;
                        flag = true;
                        break;
                    }
                default:
                    {
                        MessageBox.Show("不存在这个传递消息的key", "系统警告" , MessageBoxButtons.OK);
                        break;
                    }
            }

            this.InitPanal();
            return flag;
        }

        public UrlNodeMgr()
        {
            InitializeComponent(); 
        }

        private void BindParentID()
        {
            this.curWhere = new UrlNode()
            {
                CatalogID = this.curUrlNode.CatalogID
            };

            List<UrlNode> list = MyAction.GetUrlNode(this.curWhere);
            if (list != null)
            {
                cbxParentID.ValueMember = "ID";
                cbxParentID.DisplayMember = "Title";
                cbxParentID.DataSource = list;
            }
        }

        private void InitPanal()
        {
            if (this.curUrlNode != null)
            {
                txtTitle.Text = this.curUrlNode.Title;
                txtDesc.Text = this.curUrlNode.Desc;
                txtUrl.Text = this.curUrlNode.Url;
                if (cbxParentID.Items.Count > 0 && !string.IsNullOrEmpty(this.curUrlNode.ParentID))
                {
                    cbxParentID.SelectedIndex = cbxParentID.Items.IndexOf(this.curUrlNode.ParentID);
                }
                txtSortCode.Text = this.curUrlNode.SortCode;
            }

            this.BindParentID();
        }

        private void BindUrlNode(bool flag)
        {
            if (flag)
            {
                this.curWhere = new UrlNode()
                {
                    Title = txtTitle.Text.Trim(),
                    Desc = txtDesc.Text.Trim(),
                    SortCode = txtSortCode.Text.Trim()
                };
                if (cbxParentID.Items != null && cbxParentID.SelectedItem != null)
                {
                    this.curWhere.ParentID = cbxParentID.SelectedValue.ToString().Trim();
                }
                this.curQuery = MyAction.GetUrlNode(this.curWhere);
            }

            dgvUrlNode.DataSource = this.curQuery;
        }

        private void btnGet_Click(object sender, EventArgs e)
        {
            this.BindUrlNode(true);
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            int flag = 0;
            bool refrash = false;
            this.curUrlNode = new UrlNode()
            {
                ID = Guid.NewGuid().ToString().Trim(),
                CatalogID = this.curUrlNode.CatalogID,
                SortCode = txtSortCode.Text.Trim(),
                Title = txtTitle.Text.Trim(),
                Desc = txtDesc.Text.Trim(),
                Url = txtUrl.Text.Trim(),
                Creator = System.Environment.UserName,
                CreateTime = DateTime.Now,
                Demo = txtDemo.Text.Trim()
            };
            if (cbxParentID.Items != null && cbxParentID.SelectedItem != null)
            {
                this.curUrlNode.ParentID = cbxParentID.SelectedValue.ToString().Trim();
            }

            flag = MyAction.AddUrlNode(this.curUrlNode);
            if (flag > 0)
            {
                MessageBox.Show("添加成功！", "系统提示", MessageBoxButtons.OK);
                refrash = true;
            }
            else
            {
                MessageBox.Show("添加失败！", "系统提示", MessageBoxButtons.OK);
            }


            this.BindUrlNode(refrash);
        }

        private void btnDel_Click(object sender, EventArgs e)
        {
            if (this.curUrlNode == null)
            {
                MessageBox.Show("请选择要删除的数据！", "系统警告", MessageBoxButtons.OK);
                return;
            }

            int flag = 0;
            bool refrash = false; 

            List<UrlNode> son = MyAction.GetUrlNode(new UrlNode
                {
                    ParentID = this.curUrlNode.ID
                });
            if (son != null && son.Count > 0)
            {
                MessageBox.Show(string.Format("该节点还有{0}子节点，\n不能删除。必修先删除它的子节点！", son.Count), "", MessageBoxButtons.OK);
                return;
            }

            flag = MyAction.DelUrlNode(this.curUrlNode);
            if (flag > 0)
            {
                refrash = true;
                MessageBox.Show("删除成功！", "系统提示", MessageBoxButtons.OK);
            }
            else
            {
                MessageBox.Show("删除失败！", "系统提示", MessageBoxButtons.OK);
            }

            this.BindUrlNode(refrash);
        }

        private void btnSet_Click(object sender, EventArgs e)
        {
            int flag = 0;
            bool refrash = false;
            this.curUrlNode = new UrlNode()
            {
                ID = this.curUrlNode.ID,
                ParentID = "",
                SortCode = txtSortCode.Text.Trim(),
                Title = txtTitle.Text.Trim(),
                Desc = txtDesc.Text.Trim(),
                Url = txtUrl.Text.Trim(),
                Editor = System.Environment.UserName,
                EditTime = DateTime.Now,
                Demo = txtDemo.Text.Trim()
            };

            if (cbxParentID.SelectedItem != null)
            {
                this.curUrlNode.ParentID = cbxParentID.SelectedValue.ToString().Trim();
            } 

            flag = MyAction.SetUrlNode(this.curUrlNode);
            if (flag > 0)
            {
                refrash = true;
                MessageBox.Show("修改成功！", "系统提示", MessageBoxButtons.OK);
            }
            else
            { 
                MessageBox.Show("修改失败！", "系统提示", MessageBoxButtons.OK);
            }

            this.BindUrlNode(refrash);
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            txtTitle.Text = txtDesc.Text = txtUrl.Text =  txtSortCode.Text = txtDemo.Text = "";
            cbxParentID.SelectedIndex = 0;
            //
            this.curUrlNode = new UrlNode
            {
                CatalogID = this.curUrlNode.CatalogID
            };
        }

        private void dgvUrlNode_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0 && e.RowIndex <= dgvUrlNode.Rows.Count - 1)
            {
                DataGridViewRow row = dgvUrlNode.Rows[e.RowIndex];
                //
                this.curUrlNode = new UrlNode()
                {
                    ID = row.Cells[0].Value.ToString().Trim(), 
                    ParentID = row.Cells[2].Value.ToString().Trim(),
                    SortCode = row.Cells[3].Value.ToString().Trim(),
                    Title = row.Cells[4].Value.ToString().Trim(),
                    Desc = row.Cells[5].Value.ToString().Trim(),
                    Url = row.Cells[6].Value.ToString().Trim()
                };
                //
                txtTitle.Text = this.curUrlNode.Title;
                txtDesc.Text = this.curUrlNode.Desc;
                txtUrl.Text = this.curUrlNode.Url;
                cbxParentID.SelectedIndex = cbxParentID.Items.IndexOf(this.curUrlNode.ParentID);
                txtSortCode.Text = this.curUrlNode.SortCode;
                //
                if (this.curUrlNode != null && !string.IsNullOrEmpty(this.curUrlNode.ID))
                {
                    List<UrlNode> list = MyAction.GetUrlNode(this.curUrlNode);
                    if (list != null && list.Count > 0)
                    {
                        txtDemo.Text = list[0].Demo;
                    } 
                    // 资源回收
                    list = null;
                }
            }
        }
    }
}
